searchNodes=[{"doc":"The ExCrypto module exposes a subset of functionality from the Erlang crypto module with the goal of making it easier to include strong cryptography in your Elixir applications. This module provides functions for symmetric-key cryptographic operations using AES in GCM and CBC mode. The ExCrypto module attempts to reduce complexity by providing some sane default values for common operations.","ref":"ExCrypto.html","title":"ExCrypto","type":"module"},{"doc":"Split and decode the three parts of an encrypted payload and encode using Base.url_decode64 . Examples iex&gt; clear_text = &quot;my-clear-text&quot; iex&gt; auth_data = &quot;my-auth-data&quot; iex&gt; { :ok , aes_256_key } = ExCrypto . generate_aes_key ( :aes_256 , :bytes ) iex&gt; { :ok , { _ad , { init_vec , cipher_text , cipher_tag } } } = ExCrypto . encrypt ( aes_256_key , auth_data , clear_text ) iex&gt; { :ok , encoded_payload } = ExCrypto . encode_payload ( init_vec , cipher_text , cipher_tag ) iex&gt; assert ( String . valid? ( encoded_payload ) ) true iex&gt; { :ok , { d_init_vec , d_cipher_text , d_cipher_tag } } = ExCrypto . decode_payload ( encoded_payload ) iex&gt; assert ( d_init_vec == init_vec ) true iex&gt; assert ( d_cipher_text == cipher_text ) true iex&gt; assert ( d_cipher_tag == cipher_tag ) true","ref":"ExCrypto.html#decode_payload/1","title":"ExCrypto.decode_payload/1","type":"function"},{"doc":"Returns a clear-text string decrypted with AES256 in CBC mode. At a high level decryption using AES in CBC mode looks like this: key + init_vec + cipher_text -&gt; clear_text Examples iex&gt; clear_text = &quot;my-clear-text&quot; iex&gt; { :ok , aes_256_key } = ExCrypto . generate_aes_key ( :aes_256 , :bytes ) iex&gt; { :ok , { init_vec , cipher_text } } = ExCrypto . encrypt ( aes_256_key , clear_text ) iex&gt; { :ok , val } = ExCrypto . decrypt ( aes_256_key , init_vec , cipher_text ) iex&gt; assert ( val == clear_text ) true","ref":"ExCrypto.html#decrypt/3","title":"ExCrypto.decrypt/3","type":"function"},{"doc":"Returns a clear-text string decrypted with AES in GCM mode. At a high level decryption using AES in GCM mode looks like this: key + init_vec + auth_data + cipher_text + cipher_tag -&gt; clear_text Examples iex&gt; clear_text = &quot;my-clear-text&quot; iex&gt; auth_data = &quot;my-auth-data&quot; iex&gt; { :ok , aes_256_key } = ExCrypto . generate_aes_key ( :aes_256 , :bytes ) iex&gt; { :ok , { _ad , payload } } = ExCrypto . encrypt ( aes_256_key , auth_data , clear_text ) iex&gt; { init_vec , cipher_text , cipher_tag } = payload iex&gt; { :ok , val } = ExCrypto . decrypt ( aes_256_key , auth_data , init_vec , cipher_text , cipher_tag ) iex&gt; assert ( val == clear_text ) true","ref":"ExCrypto.html#decrypt/5","title":"ExCrypto.decrypt/5","type":"function"},{"doc":"Join the three parts of an encrypted payload and encode using Base.url_encode64 . This produces a Unicode payload string like this: init_vec &lt;&gt; cipher_text &lt;&gt; cipher_tag [ 128 bits ] &lt;&gt; [ ?? bits ] &lt;&gt; [ 128 bits ] This format is convenient to include in HTTP request bodies. It can also be used with JSON transport formats. Examples iex&gt; clear_text = &quot;my-clear-text&quot; iex&gt; auth_data = &quot;my-auth-data&quot; iex&gt; { :ok , aes_256_key } = ExCrypto . generate_aes_key ( :aes_256 , :bytes ) iex&gt; { :ok , { _ad , { init_vec , cipher_text , cipher_tag } } } = ExCrypto . encrypt ( aes_256_key , auth_data , clear_text ) iex&gt; { :ok , encoded_payload } = ExCrypto . encode_payload ( init_vec , cipher_text , cipher_tag ) iex&gt; assert ( String . valid? ( encoded_payload ) ) true","ref":"ExCrypto.html#encode_payload/3","title":"ExCrypto.encode_payload/3","type":"function"},{"doc":"Encrypt a binary with AES in CBC mode. Returns a tuple containing the initialization_vector , and cipher_text . At a high level encryption using AES in CBC mode looks like this: key + clear_text -&gt; init_vec + cipher_text Examples iex&gt; clear_text = &quot;my-clear-text&quot; iex&gt; { :ok , aes_256_key } = ExCrypto . generate_aes_key ( :aes_256 , :bytes ) iex&gt; { :ok , { _iv , cipher_text } } = ExCrypto . encrypt ( aes_256_key , clear_text ) iex&gt; assert ( is_bitstring ( cipher_text ) ) true","ref":"ExCrypto.html#encrypt/2","title":"ExCrypto.encrypt/2","type":"function"},{"doc":"Same as encrypt/4 except the initialization_vector is automatically generated. A 128 bit initialization_vector is generated automatically by encrypt/3 . It returns a tuple containing the initialization_vector , the cipher_text and the cipher_tag . Examples iex&gt; clear_text = &quot;my-clear-text&quot; iex&gt; auth_data = &quot;my-auth-data&quot; iex&gt; { :ok , aes_256_key } = ExCrypto . generate_aes_key ( :aes_256 , :bytes ) iex&gt; { :ok , { _ad , payload } } = ExCrypto . encrypt ( aes_256_key , auth_data , clear_text ) iex&gt; { _init_vec , cipher_text , cipher_tag } = payload iex&gt; assert ( is_bitstring ( cipher_text ) ) true iex&gt; assert ( bit_size ( cipher_tag ) == 128 ) true","ref":"ExCrypto.html#encrypt/3","title":"ExCrypto.encrypt/3","type":"function"},{"doc":"Encrypt a binary with AES in GCM mode. Returns a tuple containing the initialization_vector , the cipher_text and the cipher_tag . At a high level encryption using AES in GCM mode looks like this: key + init_vec + auth_data + clear_text -&gt; cipher_text + cipher_tag Examples iex&gt; clear_text = &quot;my-clear-text&quot; iex&gt; auth_data = &quot;my-auth-data&quot; iex&gt; { :ok , aes_256_key } = ExCrypto . generate_aes_key ( :aes_256 , :bytes ) iex&gt; { :ok , iv } = ExCrypto . rand_bytes ( 16 ) iex&gt; { :ok , { _ad , payload } } = ExCrypto . encrypt ( aes_256_key , auth_data , iv , clear_text ) iex&gt; { _iv , cipher_text , cipher_tag } = payload iex&gt; assert ( is_bitstring ( cipher_text ) ) true iex&gt; assert ( bit_size ( cipher_tag ) == 128 ) true","ref":"ExCrypto.html#encrypt/4","title":"ExCrypto.encrypt/4","type":"function"},{"doc":"Returns an AES key. Accepts a key_type ( :aes_128 | :aes_192 | :aes_256 ) and key_format ( :base64 | :bytes ) to determine type of key to produce. Examples iex&gt; { :ok , key } = ExCrypto . generate_aes_key ( :aes_256 , :bytes ) iex&gt; assert bit_size ( key ) == 256 true iex&gt; { :ok , key } = ExCrypto . generate_aes_key ( :aes_256 , :base64 ) iex&gt; assert String . length ( key ) == 44 true iex&gt; { :ok , key } = ExCrypto . generate_aes_key ( :aes_192 , :bytes ) iex&gt; assert bit_size ( key ) == 192 true iex&gt; { :ok , key } = ExCrypto . generate_aes_key ( :aes_192 , :base64 ) iex&gt; assert String . length ( key ) == 32 true iex&gt; { :ok , key } = ExCrypto . generate_aes_key ( :aes_128 , :bytes ) iex&gt; assert bit_size ( key ) == 128 true iex&gt; { :ok , key } = ExCrypto . generate_aes_key ( :aes_128 , :base64 ) iex&gt; assert String . length ( key ) == 24 true","ref":"ExCrypto.html#generate_aes_key/2","title":"ExCrypto.generate_aes_key/2","type":"function"},{"doc":"","ref":"ExCrypto.html#pad/2","title":"ExCrypto.pad/2","type":"function"},{"doc":"Returns a string of random where the length is equal to integer . Examples iex&gt; { :ok , rand_bytes } = ExCrypto . rand_bytes ( 16 ) iex&gt; assert ( byte_size ( rand_bytes ) == 16 ) true iex&gt; assert ( bit_size ( rand_bytes ) == 128 ) true iex&gt; { :ok , rand_bytes } = ExCrypto . rand_bytes ( 24 ) iex&gt; assert ( byte_size ( rand_bytes ) == 24 ) true iex&gt; assert ( bit_size ( rand_bytes ) == 192 ) true iex&gt; { :ok , rand_bytes } = ExCrypto . rand_bytes ( 32 ) iex&gt; assert ( byte_size ( rand_bytes ) == 32 ) true iex&gt; assert ( bit_size ( rand_bytes ) == 256 ) true","ref":"ExCrypto.html#rand_bytes/1","title":"ExCrypto.rand_bytes/1","type":"function"},{"doc":"","ref":"ExCrypto.html#rand_bytes!/1","title":"ExCrypto.rand_bytes!/1","type":"function"},{"doc":"Returns random characters. Each character represents 6 bits of entropy. Accepts an integer to determine the number of random characters to return. Examples iex&gt; rand_string = ExCrypto . rand_chars ( 24 ) iex&gt; assert ( String . length ( rand_string ) == 24 ) true iex&gt; rand_string = ExCrypto . rand_chars ( 32 ) iex&gt; assert ( String . length ( rand_string ) == 32 ) true iex&gt; rand_string = ExCrypto . rand_chars ( 44 ) iex&gt; assert ( String . length ( rand_string ) == 44 ) true","ref":"ExCrypto.html#rand_chars/1","title":"ExCrypto.rand_chars/1","type":"function"},{"doc":"Returns a random integer between low and high . Accepts two integer arguments for the low and high boundaries. The low argument must be less than the high argument. Examples iex&gt; rand_int = ExCrypto . rand_int ( 2 , 20 ) iex&gt; assert ( rand_int &gt; 1 ) true iex&gt; assert ( rand_int &lt; 21 ) true iex&gt; rand_int = ExCrypto . rand_int ( 23 , 99 ) iex&gt; assert ( rand_int &gt; 22 ) true iex&gt; assert ( rand_int &lt; 99 ) true iex&gt; rand_int = ExCrypto . rand_int ( 212 , 736 ) iex&gt; assert ( rand_int &gt; 211 ) true iex&gt; assert ( rand_int &lt; 737 ) true","ref":"ExCrypto.html#rand_int/2","title":"ExCrypto.rand_int/2","type":"function"},{"doc":"","ref":"ExCrypto.html#unpad/1","title":"ExCrypto.unpad/1","type":"function"},{"doc":"","ref":"ExCrypto.HMAC.html","title":"ExCrypto.HMAC","type":"module"},{"doc":"","ref":"ExCrypto.HMAC.html#hmac/2","title":"ExCrypto.HMAC.hmac/2","type":"function"},{"doc":"","ref":"ExCrypto.HMAC.html#hmac/3","title":"ExCrypto.HMAC.hmac/3","type":"function"},{"doc":"","ref":"ExCrypto.HMAC.html#hmac!/2","title":"ExCrypto.HMAC.hmac!/2","type":"function"},{"doc":"","ref":"ExCrypto.HMAC.html#verify_hmac/3","title":"ExCrypto.HMAC.verify_hmac/3","type":"function"},{"doc":"","ref":"ExCrypto.HMAC.html#verify_hmac/4","title":"ExCrypto.HMAC.verify_hmac/4","type":"function"},{"doc":"","ref":"ExCrypto.Hash.html","title":"ExCrypto.Hash","type":"module"},{"doc":"","ref":"ExCrypto.Hash.html#sha256/1","title":"ExCrypto.Hash.sha256/1","type":"function"},{"doc":"","ref":"ExCrypto.Hash.html#sha256!/1","title":"ExCrypto.Hash.sha256!/1","type":"function"},{"doc":"","ref":"ExCrypto.Hash.html#sha512/1","title":"ExCrypto.Hash.sha512/1","type":"function"},{"doc":"","ref":"ExCrypto.Hash.html#sha512!/1","title":"ExCrypto.Hash.sha512!/1","type":"function"},{"doc":"Use ExCrypto.Token to create unforgeable HMAC tokens that expire after a TTL. Tokens created with this module have the following properties: unforgeable expire after a given TTL may contain useful information in the payload (e.g. user_id or permissions) safe to use in HTTP headers or URLs (encoded with Base.url_encode64/1 ) Basic usage Often it's convenient to include a JSON Object as the payload. That way the data in the payload is available after the token is verified like this: iex&gt; payload = %{ &quot;user_id&quot; =&gt; 12345 } iex&gt; encoded_payload = Poison . encode! ( payload ) iex&gt; { :ok , secret } = ExCrypto . generate_aes_key ( :aes_256 , :bytes ) iex&gt; { :ok , token } = ExCrypto.Token . create ( encoded_payload , secret ) iex&gt; ttl = ( 15 * 60 ) # 15 minute TTL (in seconds) iex&gt; { :ok , verified_payload } = ExCrypto.Token . verify ( token , secret , ttl ) iex&gt; decoded_verified_payload = Poison . decode! ( verified_payload ) iex&gt; assert ( decoded_verified_payload == payload ) iex&gt; Map . get ( decoded_verified_payload , &quot;user_id&quot; ) 12345 Notes the payload is not encrypted, only base64 encoded, do not include secrets in the payload do not create a new secret each time, it must be stored and kept secret do not include the secret in the payload store the secret in the config for your app if using one global secret store the secret on a given record (e.g. user record) if using a unique secret for each user","ref":"ExCrypto.Token.html","title":"ExCrypto.Token","type":"module"},{"doc":"Generate a signed token that carries of timestamp of when it was signed. Examples iex&gt; payload = &quot;my binary payload&quot; iex&gt; { :ok , secret } = ExCrypto . generate_aes_key ( :aes_256 , :bytes ) iex&gt; { :ok , token } = ExCrypto.Token . create ( payload , secret ) iex&gt; ExCrypto.Token . is_token? ( token ) true","ref":"ExCrypto.Token.html#create/3","title":"ExCrypto.Token.create/3","type":"function"},{"doc":"Like create/3 but raises an exception on error.","ref":"ExCrypto.Token.html#create!/3","title":"ExCrypto.Token.create!/3","type":"function"},{"doc":"Check if a given binary has the correct structure to be a token. This does not mean it is a valid token, only that it has all the parts of a token. Examples iex&gt; payload = &quot;my binary payload&quot; iex&gt; { :ok , secret } = ExCrypto . generate_aes_key ( :aes_256 , :bytes ) iex&gt; { :ok , token } = ExCrypto.Token . create ( payload , secret ) iex&gt; ExCrypto.Token . is_token? ( token ) true","ref":"ExCrypto.Token.html#is_token?/1","title":"ExCrypto.Token.is_token?/1","type":"function"},{"doc":"Update the signature on an existing token. This is useful if you want to have a token that expires quickly, but only if it is not being used. For example, if you use these tokens in a cookie for authentication in a web app, you can update the token each time the user makes a request, and send the updated cookie in the response. This way a user can be logged out after N minutes of inactivity without requiring the user to supply credentials every N minutes. This is also useful if the payload is expensive to create in the first place. Another important benefit is that since the token is rotated with each request stealing a token becomes much less valuable. It's not impossible, but because the token changes with each request old tokens are only good until their TTL expires. Examples iex&gt; payload = &quot;my binary payload&quot; iex&gt; { :ok , secret } = ExCrypto . generate_aes_key ( :aes_256 , :bytes ) iex&gt; { :ok , token } = ExCrypto.Token . create ( payload , secret ) iex&gt; ExCrypto.Token . is_token? ( token ) true iex&gt; ttl = ( 15 * 60 ) # 15 minute TTL (in seconds) iex&gt; { :ok , { update_token , update_payload } } = ExCrypto.Token . update ( token , secret , ttl ) iex&gt; update_payload == payload true iex&gt; { :ok , verified_payload } = ExCrypto.Token . verify ( update_token , secret , ttl ) iex&gt; verified_payload == payload true","ref":"ExCrypto.Token.html#update/4","title":"ExCrypto.Token.update/4","type":"function"},{"doc":"Verify a token. Ensure the signature is no older than the ttl . Examples iex&gt; payload = &quot;my binary payload&quot; iex&gt; { :ok , secret } = ExCrypto . generate_aes_key ( :aes_256 , :bytes ) iex&gt; { :ok , token } = ExCrypto.Token . create ( payload , secret ) iex&gt; ExCrypto.Token . is_token? ( token ) true iex&gt; ttl = ( 15 * 60 ) # 15 minute TTL (in seconds) iex&gt; { :ok , verified_payload } = ExCrypto.Token . verify ( token , secret , ttl ) iex&gt; verified_payload == payload true","ref":"ExCrypto.Token.html#verify/4","title":"ExCrypto.Token.verify/4","type":"function"},{"doc":"Like verify/4 but raises an exception on error.","ref":"ExCrypto.Token.html#verify!/4","title":"ExCrypto.Token.verify!/4","type":"function"},{"doc":"","ref":"ExCrypto.Token.html#t:option/0","title":"ExCrypto.Token.option/0","type":"type"},{"doc":"","ref":"ExCrypto.Token.html#t:options/0","title":"ExCrypto.Token.options/0","type":"type"},{"doc":"","ref":"ExCrypto.Token.html#t:payload/0","title":"ExCrypto.Token.payload/0","type":"type"},{"doc":"","ref":"ExCrypto.Token.html#t:token/0","title":"ExCrypto.Token.token/0","type":"type"},{"doc":"","ref":"ExEntropy.html","title":"ExEntropy","type":"module"},{"doc":"","ref":"ExEntropy.html#shannon_entropy/1","title":"ExEntropy.shannon_entropy/1","type":"function"},{"doc":"Compute the Shannon entropy of a binary value. reference: http://stackoverflow.com/questions/990477/how-to-calculate-the-entropy-of-a-file https://en.wikipedia.org/wiki/Entropy_(information_theory)","ref":"ExEntropy.html#shannon_entropy/2","title":"ExEntropy.shannon_entropy/2","type":"function"},{"doc":"API module for public-key infrastructure. Description Mostly wrappers Erlang' :public_key module, to help simplify using public/private key encryption in Elixir.","ref":"ExPublicKey.html","title":"ExPublicKey","type":"module"},{"doc":"","ref":"ExPublicKey.html#decrypt_private/3","title":"ExPublicKey.decrypt_private/3","type":"function"},{"doc":"","ref":"ExPublicKey.html#decrypt_public/3","title":"ExPublicKey.decrypt_public/3","type":"function"},{"doc":"","ref":"ExPublicKey.html#encrypt_private/3","title":"ExPublicKey.encrypt_private/3","type":"function"},{"doc":"","ref":"ExPublicKey.html#encrypt_public/3","title":"ExPublicKey.encrypt_public/3","type":"function"},{"doc":"","ref":"ExPublicKey.html#generate_key/0","title":"ExPublicKey.generate_key/0","type":"function"},{"doc":"","ref":"ExPublicKey.html#generate_key/1","title":"ExPublicKey.generate_key/1","type":"function"},{"doc":"","ref":"ExPublicKey.html#generate_key/2","title":"ExPublicKey.generate_key/2","type":"function"},{"doc":"Generate a new key. Note: To ensure Backwards compatibility when generating rsa keys on OTP &lt; 20, we fall back to openssl via System.cmd. Example { :ok , rsa_priv_key } = ExPublicKey . generate_key ( :rsa , 2048 )","ref":"ExPublicKey.html#generate_key/3","title":"ExPublicKey.generate_key/3","type":"function"},{"doc":"","ref":"ExPublicKey.html#generate_key/4","title":"ExPublicKey.generate_key/4","type":"function"},{"doc":"Loads PEM string from the specified file path and returns a ExPublicKey.RSAPrivateKey or a ExPublicKey.RSAPublicKey key. Optionally, a passphrase can be given to decode the PEM certificate. Examples { :ok , key } = ExPublicKey . load ( &quot;/file/to/cert.pem&quot; ) { :ok , key } = ExPublicKey . load ( &quot;/file/to/cert.pem&quot; , &quot;pem_password&quot; )","ref":"ExPublicKey.html#load/2","title":"ExPublicKey.load/2","type":"function"},{"doc":"Loads PEM string from the specified file path and returns a ExPublicKey.RSAPrivateKey or a ExPublicKey.RSAPublicKey key. Optionally, a passphrase can be given to decode the PEM certificate. Identical to ExPublicKey.load/2 , except that load! raises an ExCrypto.Error when an exception occurs. Examples key = ExPublicKey . load ( &quot;/file/to/cert.pem&quot; ) key = ExPublicKey . load ( &quot;/file/to/cert.pem&quot; , &quot;pem_password&quot; )","ref":"ExPublicKey.html#load!/2","title":"ExPublicKey.load!/2","type":"function"},{"doc":"Converts a PEM string into an ExPublicKey.RSAPrivateKey or an ExPublicKey.RSAPublicKey key. Optionally, a passphrase can be given to decode the PEM certificate. Examples { :ok , key } = ExPublicKey . loads ( pem_string ) { :ok , key } = ExPublicKey . loads ( pem_string , &quot;pem_password&quot; )","ref":"ExPublicKey.html#loads/2","title":"ExPublicKey.loads/2","type":"function"},{"doc":"Converts a PEM string into an ExPublicKey.RSAPrivateKey or an ExPublicKey.RSAPublicKey key. Identical to ExPublicKey.loads/2 , except that loads! raises an ExCrypto.Error when an exception occurs. Example key = ExPublicKey . loads! ( pem_string )","ref":"ExPublicKey.html#loads!/2","title":"ExPublicKey.loads!/2","type":"function"},{"doc":"","ref":"ExPublicKey.html#normalize_error/3","title":"ExPublicKey.normalize_error/3","type":"function"},{"doc":"Encode a key into a PEM string. To decode, use ExPublicKey.loads/1 Example { :ok , pem_string } = ExPublicKey . pem_encode ( key )","ref":"ExPublicKey.html#pem_encode/1","title":"ExPublicKey.pem_encode/1","type":"function"},{"doc":"Extract the public part of a private string and return the results as a ExPublicKey.RSAPublicKey struct. Example { :ok , rsa_pub_key } = ExPublicKey . public_key_from_private_key ( rsa_priv_key )","ref":"ExPublicKey.html#public_key_from_private_key/1","title":"ExPublicKey.public_key_from_private_key/1","type":"function"},{"doc":"","ref":"ExPublicKey.html#sign/2","title":"ExPublicKey.sign/2","type":"function"},{"doc":"","ref":"ExPublicKey.html#sign/3","title":"ExPublicKey.sign/3","type":"function"},{"doc":"","ref":"ExPublicKey.html#verify/3","title":"ExPublicKey.verify/3","type":"function"},{"doc":"","ref":"ExPublicKey.html#verify/4","title":"ExPublicKey.verify/4","type":"function"},{"doc":"","ref":"ExPublicKey.RSAPrivateKey.html","title":"ExPublicKey.RSAPrivateKey","type":"module"},{"doc":"","ref":"ExPublicKey.RSAPrivateKey.html#as_sequence/1","title":"ExPublicKey.RSAPrivateKey.as_sequence/1","type":"function"},{"doc":"","ref":"ExPublicKey.RSAPrivateKey.html#decode_der/1","title":"ExPublicKey.RSAPrivateKey.decode_der/1","type":"function"},{"doc":"","ref":"ExPublicKey.RSAPrivateKey.html#encode_der/1","title":"ExPublicKey.RSAPrivateKey.encode_der/1","type":"function"},{"doc":"","ref":"ExPublicKey.RSAPrivateKey.html#from_sequence/1","title":"ExPublicKey.RSAPrivateKey.from_sequence/1","type":"function"},{"doc":"","ref":"ExPublicKey.RSAPrivateKey.html#get_fingerprint/2","title":"ExPublicKey.RSAPrivateKey.get_fingerprint/2","type":"function"},{"doc":"","ref":"ExPublicKey.RSAPrivateKey.html#get_public/1","title":"ExPublicKey.RSAPrivateKey.get_public/1","type":"function"},{"doc":"","ref":"ExPublicKey.RSAPrivateKey.html#t:t/0","title":"ExPublicKey.RSAPrivateKey.t/0","type":"type"},{"doc":"","ref":"ExPublicKey.RSAPublicKey.html","title":"ExPublicKey.RSAPublicKey","type":"module"},{"doc":"","ref":"ExPublicKey.RSAPublicKey.html#as_sequence/1","title":"ExPublicKey.RSAPublicKey.as_sequence/1","type":"function"},{"doc":"","ref":"ExPublicKey.RSAPublicKey.html#decode_der/2","title":"ExPublicKey.RSAPublicKey.decode_der/2","type":"function"},{"doc":"","ref":"ExPublicKey.RSAPublicKey.html#encode_der/1","title":"ExPublicKey.RSAPublicKey.encode_der/1","type":"function"},{"doc":"","ref":"ExPublicKey.RSAPublicKey.html#from_der_encoded_0/1","title":"ExPublicKey.RSAPublicKey.from_der_encoded_0/1","type":"function"},{"doc":"","ref":"ExPublicKey.RSAPublicKey.html#from_sequence/1","title":"ExPublicKey.RSAPublicKey.from_sequence/1","type":"function"},{"doc":"","ref":"ExPublicKey.RSAPublicKey.html#get_fingerprint/2","title":"ExPublicKey.RSAPublicKey.get_fingerprint/2","type":"function"},{"doc":"","ref":"ExPublicKey.RSAPublicKey.html#t:t/0","title":"ExPublicKey.RSAPublicKey.t/0","type":"type"},{"doc":"","ref":"ExCrypto.Error.html","title":"ExCrypto.Error","type":"exception"},{"doc":"Callback implementation for Exception.message/1 .","ref":"ExCrypto.Error.html#message/1","title":"ExCrypto.Error.message/1","type":"function"},{"doc":"ExCrypto The goal of ExCrypto and ExPublicKey is to expose a subset of the functionality from the Erlang modules crypto and public_key so that writing secure Elixir applications is a little bit easier without being overwhelming. In many functions some sane defaults are provided to decrease the complexity of implementing those functions in your own code. Get from hex.pm: https://hex.pm/packages/ex_crypto Checkout the docs at: https://hexdocs.pm/ex_crypto/readme.html","ref":"readme.html","title":"ExCrypto","type":"extras"},{"doc":"","ref":"readme.html#build-status","title":"ExCrypto - Build Status","type":"extras"},{"doc":"The ExPublicKey module provides functions for working with RSA public/private key operations. There are a couple common uses for public-key cryptography: Authenticate a message Transmit a shared secret (session key) Authenticate a message The goal of ExPublicKey.sign and ExPublicKey.verify is that the recipient of a message can identify the sender. This can be accomplished as follows: Note: assume the message is a JSON payload. Sender serialize the JSON to a string hash, time-stamp, and sign with your private-key Receiver ensure the time-stamp is within the current window verify the signature with the sender's public-key Here are the steps in order: # load the RSA keys from a file on disk rsa_priv_key = ExPublicKey . load! ( &quot;/path/to/private_key.pem&quot; ) rsa_pub_key = ExPublicKey . load! ( &quot;/path/to/public_key.pem&quot; ) # create the message JSON msg = %{ &quot;name_first&quot; =&gt; &quot;Chuck&quot; , &quot;name_last&quot; =&gt; &quot;Norris&quot; } # serialize the JSON msg_serialized = Poison . encode! ( msg ) # generate time-stamp ts = DateTime . utc_now |&gt; DateTime . to_unix # add a time-stamp ts_msg_serialized = &quot; \#{ ts } | \#{ msg_serialized } &quot; # generate a secure hash using SHA256 and sign the message with the private key { :ok , signature } = ExPublicKey . sign ( ts_msg_serialized , rsa_priv_key ) # combine payload payload = &quot; \#{ ts } | \#{ msg_serialized } | \#{ Base . url_encode64 signature } &quot; IO . puts payload # pretend transmit the message... # pretend receive the message... # break up the payload parts = String . split ( payload , &quot;|&quot; ) recv_ts = Enum . fetch! ( parts , 0 ) recv_msg_serialized = Enum . fetch! ( parts , 1 ) { :ok , recv_sig } = Enum . fetch! ( parts , 2 ) |&gt; Base . url_decode64 # pretend ensure the time-stamp is not too old (or from the future)... # it should probably no more than 5 minutes old, and no more than 15 minutes in the future # verify the signature { :ok , sig_valid } = ExPublicKey . verify ( &quot; \#{ recv_ts } | \#{ recv_msg_serialized } &quot; , recv_sig , rsa_pub_key ) assert ( sig_valid ) # un-serialize the JSON recv_msg_unserialized = Poison.Parser . parse! ( recv_msg_serialized ) assert ( msg == recv_msg_unserialized ) Note: this example is similar to the test &quot;sign and verify a JSON payload&quot; in test/ex_public_key_test.exs . Transmit a shared secret (in progress) Load the keys from PEM format files First load the public/private RSA keys from disk: iex(1)&gt; { :ok , rsa_private_key } = ExPublicKey . load ( &quot;/tmp/test_rsa_private_key.pem&quot; ) { :ok , % ExPublicKey.RSAPrivateKey { ... } } iex(2)&gt; { :ok , rsa_public_key } = ExPublicKey . load ( &quot;/tmp/test_rsa_public_key.pem&quot; ) { :ok , % ExPublicKey.RSAPublicKey { ... } } Sign with RSA private key To create a signature with the RSAPrivateKey like this: iex(3)&gt; message = &quot;A very important message.&quot; &quot;A very important message.&quot; ex ( 4 ) &gt; { :ok , signature } = ExPublicKey . sign ( message , rsa_private_key ) { :ok , &lt;&lt; ... &gt;&gt; } Verify signature with RSA public key iex(5)&gt; { :ok , valid } = ExPublicKey . verify ( message , signature , rsa_public_key ) { :ok , true } Encrypt with RSA public key iex(6)&gt; clear_text = &quot;A super important message&quot; &quot;A super important message&quot; iex(7)&gt; { :ok , cipher_text } = ExPublicKey . encrypt_public ( clear_text , rsa_public_key ) { :ok , &quot;Lmbv...HQ==&quot; } Decrypt with RSA private key iex(8)&gt; { :ok , decrypted_clear_text } = ExPublicKey . decrypt_private ( cipher_text , rsa_private_key ) { :ok , &quot;A super important message&quot; }","ref":"readme.html#using-expublickey","title":"ExCrypto - Using ExPublicKey","type":"extras"},{"doc":"The ExCrypto module provides relatively functions for AES cryptography operations. Generate AES keys Generate a new 128 bit AES key like this: iex(1)&gt; { :ok , aes_128_key } = ExCrypto . generate_aes_key ( :aes_128 , :bytes ) { :ok , &lt;&lt; ... &gt;&gt; } Often it's more convenient to handle the key as a base64 encoded string and you can generate a new key, already encoded as a base64 unicode string like this: iex(2)&gt; { :ok , aes_128_key } = ExCrypto . generate_aes_key ( :aes_128 , :base64 ) { :ok , &quot;deGqaW9gP1_0WlSomf2pZDzeyGcitSmfXYu7ygTsypsrSmvTVfl7ANQsTWc30TP9IftiBnmDlqkuU1ARzAN82Fo1NMJhvVi3iWkzYe9yusm0s3ymUh4Hs2O7oZCgJeavFwuHgrpk_79nyfe3HkSNoAVjNWv0ImOmLyClrPIa3qk=&quot; } You can also generate 192/256 bit AES keys like this: iex(3)&gt; { :ok , aes_192_key } = ExCrypto . generate_aes_key ( :aes_192 , :base64 ) { :ok , &quot;P173Su55_bFR4WEf4SmKC4yKAX-IT9-83rbS6RSIPxEHf7uTEvyr969C3ZCkbSh5dJrWd35zjYQM-l5DpGzdIztxCqvN9myGYUdrfn9D2PRh9Y7XgQWRqYJ6FE67EHcNgJWrxEQ_HRt5jBczoY-34AZAN3RVcVqXrwGZw6ISJcyKVc30nJOBS9N4QeQWw2bPrppfzA43-_hAVfjEKCUyPzi2zlG2WUsaeKS4vOOmVAzkC0IPbONqVtzlxiFwbr7I&quot; } iex(4)&gt; { :ok , aes_256_key } = ExCrypto . generate_aes_key ( :aes_256 , :base64 ) { :ok , &quot;Bs_BzhuwseEA8ZUvuEY0mq9Rmlv6cSoU_RaYD14Q62HiN_kJ4FiaW0YYppf1ffYPQ56xuitxQtYAnaeP-Q5l1WPh5aExdwCG_PUm5g-MlOUA1XSSP2RvuQqAiHzazIzjGVSIcl0Gr7TSLPOoIQrPshMNaA4j3SGZ3lAOqO1quvXtDn-9Sxwr5dwV7VzOIvXRwb0GbZeYp8lnVJgeqHl8cEhUTfT_h9Pm7tU2CFeHZCDK8ntFT_t4q6VlcBcvw_Pj3CGcVSmpmCHMKW1brt6jXGBijqSTdbjYDZnCx2Q44VoYqMMZ1U2GnVyjc-ZuwugwGGqQ7UEqV_TOMjbK6Oxx-Q==&quot; } In both examples the :bytes atom can be substituted for :base64 if you wish to receive your key as a bitstring rather than as a base64 encoded unicode string. As you can see the keys grow longer in order of bit length. A 128 bit key is more than sufficient for most applications but if you are slightly more paranoid than average use a 192 bit key. If your paranoia knows no bounds or you are protecting state secrets from nation-state owned quantum computers use a 256 bit key. If you are concerned about hyper-advanced aliens with quantum computers you might need a longer key. Enterprise grade keys such as this can be generated upon request in the context of a consulting agreement. For this application we recommend at least a 612 bit key.","ref":"readme.html#using-excrypto","title":"ExCrypto - Using ExCrypto","type":"extras"}]